@@ -1,22 +1,23 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 #########################################################
-## Thai Language Toolkit : version  1.3.7
+## Thai Language Toolkit : version  1.3.9
 ## Chulalongkorn University
 ## word_segmentation, syl_segementation written by Wirote Aroonmanakun
 ## Implemented :
 ##      chunk, ner_tag, segment, word_segment, syl_segment, word_segment_mm, word_segment_nbest,
-##      g2p, th2ipa, th2roman, pos_tag, pos_tag_wordlist, 
+##      g2p, th2ipa, th2roman, spell_variants, pos_tag, pos_tag_wordlist, 
 ##      read_thaidict, reset_thaidict, check_thaidict
 ##      spell_candidates,
 #########################################################
-import tltk
 
 import re
 import math
 import os
 from copy import deepcopy
 from collections import defaultdict
+from itertools import product
+
 #import pkg_resources
 import pickle
 from nltk.tag.perceptron import PerceptronTagger
@@ -68,6 +69,7 @@ def check_thaidict(Word):
         return(0)
     
     
+
 ####################################################################
 ##  spelling correction modified from Peter Norvig  http://norvig.com/spell-correct.html
 ####################################################################
@@ -102,6 +104,9 @@ def edits1(word):
 def edits2(word): 
     return list(e2 for e1 in edits1(word) for e2 in edits1(e1))
 
+
+
+
 ##########################################################################
 ## POS tagging using nltk.tag.perceptron
 #########################################################################
@@ -1017,6 +1022,7 @@ def ProbPhone(p,pw,w,nw):
 #        print('Bigram2',PhSBigram[(w,nw,p)])
         p2 = p2 + (1. + math.log(PhSBigram[(w,nw,p)])) / (1. + math.log(FrmSBigram[(w,nw)])) * 0.75
     if PhSUnigram[(w,p)] > 0.:
+#        print('Unigram',PhSUnigram[(w,p)])
         p1 = (1 + math.log(PhSUnigram[(w,p)])) / (1. + math.log(FrmSUnigram[w]))
 
 ### get abstract form of sounds
@@ -1026,52 +1032,11 @@ def ProbPhone(p,pw,w,nw):
     abs_p = re.sub(r"[^aeio@OuxU]","C",abs_p)
     if AbsUnigram[(abs_w,abs_p)] > 0.:
         p0 = (1 + math.log(AbsUnigram[(abs_w,abs_p)])) / (1. + math.log(AbsFrmSUnigram[abs_w]))
-    prob =  0.8*p3 + 0.16*p2 + 0.03*p1 + 0.001*p0 + 0.00000000001
+#        print('AbsUnigram',p0)
+    prob =  0.8*p3 + 0.16*p2 + 0.03*p1 + 0.00001*p0 + 0.00000000001
+#    prob =  0.8*p3 + 0.16*p2 + 0.03*p1 + 0.00000000001
     return(prob)
 
-def read_PhSTrigram(File):
-    global PhSTrigram
-    global FrmSTrigram
-    global PhSBigram
-    global FrmSBigram
-    global PhSUnigram
-    global FrmSUnigram
-    global AbsUnigram
-    global AbsFrmSUnigram
-    
-    PhSTrigram = defaultdict(float)
-    FrmSTrigram = defaultdict(float)
-    PhSBigram = defaultdict(float)
-    FrmSBigram = defaultdict(float)
-    PhSUnigram = defaultdict(float)
-    FrmSUnigram = defaultdict(float)
-    AbsUnigram = defaultdict(float)
-    AbsFrmSUnigram = defaultdict(float)
-    
-    IFile = open(File,'r',encoding='cp874')
-    for line in IFile.readlines():
-        line = line.rstrip()
-        line = re.sub(r"<w>","|",line)
-        (x, ct) = line.split('\t')
-        (fs,p) = x.split('/')
-        (x1,x2,x3) = fs.split('-')
-#        print('read ',x1,x2,x3,p,ct)
-        PhSTrigram[(x1,x2,x3,p)] += float(ct)
-        FrmSTrigram[(x1,x2,x3)] += float(ct)
-        PhSBigram[(x1,x2,p)] += float(ct)
-        FrmSBigram[(x1,x2)] += float(ct)
-        PhSBigram[(x2,x3,p)] += float(ct)
-        FrmSBigram[(x2,x3)] += float(ct)
-        PhSUnigram[(x2,p)] += float(ct)
-        FrmSUnigram[x2] += float(ct)
-        abs_x2 = re.sub(r"[่้๊๋]","",x2)
-        abs_x2 = re.sub(r"[ก-ฮ]","C",abs_x2)
-        abs_p = re.sub(r"[0-9]","",p)
-        abs_p = re.sub(r"[^aeio@OuxU]","C",abs_p)
-#        print(x2,'=>',abs_x2,':',p,'=>',abs_p)
-        AbsUnigram[(abs_x2,abs_p)] += float(ct)
-        AbsFrmSUnigram[abs_x2] += float(ct)
-    IFile.close()
 
     
 def th2ipa(txt):
@@ -1101,6 +1066,7 @@ def th2roman(txt):
             tran = tran.replace(k, v)
         tran = re.sub(r"([aeiou])j",r"\1i",tran)
         tran = tran.replace('j','y')
+        tran = re.sub(r"c([^h])",r"ch\1",tran)
         tran = re.sub(r"\-([^aeiou])",r"\1",tran)
         out += tran+'<s/>'
     return(out)
@@ -1108,6 +1074,68 @@ def th2roman(txt):
 ### end of modules used in g2p  ###############    
 ##############################################################################################################
 
+    
+####################################################################
+##  spelling variant of the same pronunciation
+####################################################################
+def spell_variants(wrd,InDict='no',Karan='exclude'):
+    global SYLVAR
+
+    tmp = g2p(wrd)
+    tmp = tmp.replace('<s/>','')
+    tmp = tmp.replace('|','~')
+    (syls, trans) = tmp.split('<tr/>')
+    var = []
+    out=[]
+    tran_lst = trans.split('~')
+    syl_lst = syls.split('~')
+    for i in range(len(tran_lst)):
+        if tran_lst[i] != '':
+            if Karan == 'exclude':
+                SYLVAR[tran_lst[i]] = [value for value in SYLVAR[tran_lst[i]] if '์' not in value]
+            if syl_lst[i] in SYLVAR[tran_lst[i]]:
+                var.append(SYLVAR[tran_lst[i]])
+            else:    
+                var.append([syl_lst[i]])
+#    print('var',var)       
+    la = var[0]
+    for i in range(len(var)-1):
+#        print('product',la,var[i+1])
+        x = product(la,var[i+1])
+        la = []
+        for a in x:
+            la.append(''.join(a))
+    for x in la:
+#        print(''.join(x))
+        altwrd = ''.join(x)
+        if InDict == 'yes':        
+            if altwrd in TDICT:
+                out.append(altwrd)
+        else:        
+            out.append(altwrd)
+ 
+#    if len(var) == 2:
+#        all_var = product(var[0],var[1])    
+#    elif len(var) == 3:    
+#        all_var = product(var[0],var[1],var[2])
+#    elif len(var) == 4:    
+#        all_var = product(var[0],var[1],var[2],var[3])
+#    elif len(var) == 5:    
+#        all_var = product(var[0],var[1],var[2],var[3],var[4])
+#    elif len(var) == 6:    
+#        all_var = product(var[0],var[1],var[2],var[3],var[4],var[5])
+#    for x in all_var:
+#        print('xxx',x)
+#        altwrd = ''.join(x)
+#        if InDict == 'yes':        
+#            if altwrd in TDICT:
+#                out.append(altwrd)
+#        else:        
+#            out.append(altwrd)
+    return(out)        
+
+
+
 
 #############################################################################################################
 ###  Word Segmentation for Thai texts
@@ -1165,9 +1193,9 @@ def wordseg_colloc(Input):
 #        SylLst = inx.split(SylSep)
         EndOfSent = len(SylLst)
         ######### Gen unknown word by set each syllable as a potential word
-#        gen_unknown_thaiw(SylLst)
-        for i in range(EndOfSent):
-            chart[i][i+1] = [SylLst[i]]
+        gen_unknown_w(SylLst)
+#        for i in range(EndOfSent):
+#            chart[i][i+1] = [SylLst[i]]
         eng_abbr(SylLst)    
         ############################################################
         for i in range(EndOfSent):
@@ -1510,13 +1538,13 @@ def compute_colloc(stat,w1,w2):
         Count[w1] +=1
         Count[w2] +=1 
         TotalWord +=2
-    
+#    print(w1,w2,Count[w1],Count[w2],BiCount[(w1,w2)],TotalWord)    
 ###########################
 ##  Mutual Information
 ###########################
     if stat == "mi":
         mi = float(BiCount[(w1,w2)] * TotalWord) / float((Count[w1] * Count[w2]))
-        value = math.log(mi,2)
+        value = abs(math.log(mi,2))
 #########################
 ### Compute Chisquare
 ##########################
@@ -1532,6 +1560,23 @@ def compute_colloc(stat,w1,w2):
     
 ##############################################################################    
 ########  create each unit (char/syllable) as a possible edge for chart parsing
+def gen_unknown_w(SylLst):
+    global chart
+    global EndOfSent
+
+    for i in range(EndOfSent):
+        chart[i][i+1] = [SylLst[i]]
+        if SylLst[i] not in TDICT:
+            for j in range(i+1,EndOfSent):
+                if SylLst[j] not in TDICT:
+                    newwrd = ''.join(SylLst[i:j+1])
+                    chart[i][j+1] = [newwrd]
+#                    print('Create',SylLst[i],SylLst[j],newwrd)
+                else:
+                    break    
+    return(1)
+
+
 def gen_unknown_thaiw(SylLst):
     global chart
     global EndOfSent
@@ -1863,7 +1908,7 @@ def read_sylpattern(Filename):
 #    for f in PRON:
 #        for x in PRON[f]:
 #            print(f,x)
-    stable['X'] = { 'ก' : 'k', 'ข' : 'kh' , 'ค' : 'kh', 'ฆ' : 'kh', 'ง' : 'N', 'จ' : 'c', 'ฉ' : 'ch', 'ช' : 'ch', 'ซ' : 's', 'ฌ' : 'ch','ญ' : 'j','ฎ' : 'd','ฏ' : 't','ฐ' : 'th','ฑ' : 'th','ฒ' : 'th','ณ' : 'n','ด' : 'd','ต' : 't','ถ' : 'th','ท' : 'th','ธ' : 'th','น' : 'n','บ' : 'b','ป' : 'p','ผ' : 'ph','ฝ' : 'f','พ' : 'ph','ฟ' : 'f','ภ' : 'ph','ม' : 'm','ย' : 'j','ร' : 'r','ฤ' : 'r','ล' : 'l','ฦ' : 'l','ว' : 'w','ศ' : 's','ษ' : 's','ส' : 's','ห' : 'h','ฬ' : 'l','อ' : '?','ฮ' : 'h' }
+    stable['X'] = { 'ก' : 'k', 'ข' : 'kh' , 'ฃ':'kh', 'ค' : 'kh', 'ฅ' : 'kh','ฆ' : 'kh', 'ง' : 'N', 'จ' : 'c', 'ฉ' : 'ch', 'ช' : 'ch', 'ซ' : 's', 'ฌ' : 'ch','ญ' : 'j','ฎ' : 'd','ฏ' : 't','ฐ' : 'th','ฑ' : 'th','ฒ' : 'th','ณ' : 'n','ด' : 'd','ต' : 't','ถ' : 'th','ท' : 'th','ธ' : 'th','น' : 'n','บ' : 'b','ป' : 'p','ผ' : 'ph','ฝ' : 'f','พ' : 'ph','ฟ' : 'f','ภ' : 'ph','ม' : 'm','ย' : 'j','ร' : 'r','ฤ' : 'r','ล' : 'l','ฦ' : 'l','ว' : 'w','ศ' : 's','ษ' : 's','ส' : 's','ห' : 'h','ฬ' : 'l','อ' : '?','ฮ' : 'h' }
     stable['Y'] = { 'ก' : 'k', 'ข' : 'k' , 'ค' : 'k', 'ฆ' : 'k', 'ง' : 'N', 'จ' : 't', 'ฉ' : '-', 'ช' : 't', 'ซ' : 't', 'ฌ' : '-','ญ' : 'n','ฎ' : 't','ฏ' : 't','ฐ' : 't','ฑ' : 't','ฒ' : 't','ณ' : 'n','ด' : 't', 'ต' : 't','ถ' : 't','ท' : 't','ธ' : 't','น' : 'n','บ' : 'p','ป' : 'p','ผ' : '-','ฝ' : '-','พ' : 'p','ฟ' : 'p','ภ' : 'p','ม' : 'm','ย' : 'j','ร' : 'n','ฤ' : '-','ล' : 'n','ฦ' : '-','ว' : 'w','ศ' : 't','ษ' : 't','ส' : 't' ,'ห' : '-','ฬ' : 'n','อ' : '-','ฮ' : '-' }
 
     stable['A'] = stable['X']
@@ -1924,7 +1969,6 @@ def read_sylpattern(Filename):
 ##########  Read syllanle dict, pronunciation not conformed to sylrule is specified here
 def read_syldict(Filename):
     global PRON
-#    PRON = defaultdict(list)
     
     file1 = open(Filename,'r',encoding='cp874')
     for line in file1:
@@ -1936,9 +1980,62 @@ def read_syldict(Filename):
         PRON[tmp[0]].append(tmp[1])
     return(1)
 
+##########  Read syllable pattersn and pronunciation table
+def read_PhSTrigram(File):
+    global PhSTrigram
+    global FrmSTrigram
+    global PhSBigram
+    global FrmSBigram
+    global PhSUnigram
+    global FrmSUnigram
+    global AbsUnigram
+    global AbsFrmSUnigram
+    
+    PhSTrigram = defaultdict(float)
+    FrmSTrigram = defaultdict(float)
+    PhSBigram = defaultdict(float)
+    FrmSBigram = defaultdict(float)
+    PhSUnigram = defaultdict(float)
+    FrmSUnigram = defaultdict(float)
+    AbsUnigram = defaultdict(float)
+    AbsFrmSUnigram = defaultdict(float)
+    
+    IFile = open(File,'r',encoding='cp874')
+    for line in IFile.readlines():
+        line = line.rstrip()
+        line = re.sub(r"<w>","|",line)
+        (x, ct) = line.split('\t')
+        (fs,p) = x.split('/')
+        (x1,x2,x3) = fs.split('-')
+#        print('read ',x1,x2,x3,p,ct)
+        PhSTrigram[(x1,x2,x3,p)] += float(ct)
+        FrmSTrigram[(x1,x2,x3)] += float(ct)
+        PhSBigram[(x1,x2,p)] += float(ct)
+        FrmSBigram[(x1,x2)] += float(ct)
+        PhSBigram[(x2,x3,p)] += float(ct)
+        FrmSBigram[(x2,x3)] += float(ct)
+        PhSUnigram[(x2,p)] += float(ct)
+        FrmSUnigram[x2] += float(ct)
+        abs_x2 = re.sub(r"[่้๊๋]","",x2)
+        abs_x2 = re.sub(r"[ก-ฮ]","C",abs_x2)
+        abs_p = re.sub(r"[0-9]","",p)
+        abs_p = re.sub(r"[^aeio@OuxU]","C",abs_p)
+#        print(x2,'=>',abs_x2,':',p,'=>',abs_p)
+        AbsUnigram[(abs_x2,abs_p)] += float(ct)
+        AbsFrmSUnigram[abs_x2] += float(ct)
+    IFile.close()
+
+#### read syllable variants file
+def read_sylvar(Filename):
+    global SYLVAR
+
+    fileObject = open(Filename,'rb')  
+    SYLVAR = pickle.load(fileObject)
+    fileObject.close()
+    return(1)
+
 ####  read trigram statistics file
 def read_stat(Filename):
-
     global TriCount
     global BiCount
     global Count
@@ -1963,6 +2060,7 @@ def read_stat(Filename):
     BiType.clear()
     Type.clear()
 
+
     fileObject = open(Filename,'rb')  
     TriCount = pickle.load(fileObject)
     for (X,Y,Z) in TriCount:
@@ -2044,8 +2142,10 @@ def initial():
     global TDICT
     global PRON
     global CProb
+    global SYLVAR
 
     PRON = defaultdict(list)
+    SYLVAR = defaultdict(list)
 #    PRON = {}    
     TDICT = {}
     CProb = defaultdict(float)
@@ -2065,16 +2165,20 @@ def initial():
     read_syldict(ATA_PATH +  '/thaisyl.dict')
     read_stat(ATA_PATH + '/sylseg.3g')
     read_thdict(ATA_PATH +  '/thdict')
+    read_sylvar(ATA_PATH + '/sylform_var.pick')
 
     read_PhSTrigram(ATA_PATH +  '/PhSTrigram.sts')
 
     return(1)
 
 
-############ END OF GENERAL MODULES ##########################################################################
-
+############ END OF GENERAL MODULES 
 initial()
 
+
+##########################################################################
+## testing area
+
 #import time
 
 #start = time.time()
@@ -2083,17 +2187,20 @@ initial()
 #end = time.time()
 #print(end - start)
 
-
-
-################# testing area #######################################
-#print(th2roman('เด็กอยากไปโรงเรียน วันนี้ที่นี่เท่านั้น วันอื่นไม่ไป วันไหนก็ไม่ไป พรุ่งนี้ก็ไม่'))
+################# te
+# sting area #######################################
+#print(g2p('มาหรรษา'))
+#print(len(SYLVAR))    
+#print(spell_variants('โควิด',Karan='exclude'))
+#cprint(g2p('ราคาค่าตัววันนี้ไอซ์แลนด์'))
+#print(word_segment('คอยสังเกตดูอาการของตัวเองด้วยนะซาศโฏะจัง'))
+#print(th2roman('เด็กอยากไปจุฬาเช้านี้ วันนี้ที่นี่เท่านั้น วันอื่นไม่ไป วันไหนก็ไม่ไป พรุ่งนี้ก็ไม่'))
 #print(g2p('ใคร'))
 #print(word_segment('ห ลั ก ก า ร เ รื่ อ ง นี้'))
 #print(ner_tag("สำนักงานเขตจตุจักรชี้แจงว่า ได้นำป้ายประกาศเตือนปลิงไปปักตามแหล่งน้ำ ในเขตอำเภอเมือง จังหวัดอ่างทอง หลังจากนายสุกิจ อายุ 65 ปี ถูกปลิงกัดแล้วไม่ได้ไปพบแพทย์"))
 #print(th2ipa('หลักความประพฤติ'))
 #print(ner_tag('สำนักงานเขตจตุจักรชี้แจงว่า ได้นำป้ายประกาศเตือนปลิงไปปักตามแหล่งน้ำ ในเขตอำเภอเมือง จังหวัดอ่างทอง หลังจากนายสุกิจ อายุ 65 ปี ถูกปลิงกัดแล้วไม่ได้ไปพบแพทย์'))
 #print(segment('ที่ทำให้ประชาชนชาวไทยเกิดความไม่สบายใจ'))
-#print(g2p('ราคาค่าตัววันนี้ไอซ์แลนด์'))
 #print(word_segment_mm('นายกรัฐมนตรีกล่าวกับคนขับรถประจำทางหลวงสายสองว่า อยากวิงวอนให้ใช้ความรอบคอบอย่าหลงเชื่อคำชักจูงหรือปลุกระดมของพวกหัวรุนแรงจากทางการไฟฟ้า'))
 #print(g2p('นายกรัฐมนตรีกล่าวกับคนขับรถประจำทางหลวงสายสองว่า อยากวิงวอนให้ใช้ความรอบคอบอย่าหลงเชื่อคำชักจูงหรือปลุกระดมของพวกหัวรุนแรงจากทางการไฟฟ้า'))
 
